import datetime
import uuid

from nsj_gcf_utils.db_adapter2 import DBAdapter2
from nsj_rest_lib.exception import (
    ConflictException,
    NotFoundException,
)


class AnexoDAO:

    def __init__(self, db_adapter: DBAdapter2):
        self._db = db_adapter

    def get(self, id: uuid.UUID):
        sql = """
        with anexo_modulo_por_doc as (
            select
                am.documentoged, max(am.anexomodulo::text)::uuid as anexomodulo
            from
                ns.anexosmodulos as am
            group by
                am.documentoged
        )
        select
            dg.data, dg.nome, dg.documento, dg.mimetype, dg.documentoged, am.descricao, am.tipo, am.id_modulodoanexo, am.modulodoanexo
        from
            ns.documentosged as dg
            left join anexo_modulo_por_doc as amd on (amd.documentoged = dg.documentoged)
            left join ns.anexosmodulos as am on (am.anexomodulo = amd.anexomodulo)
        where
            dg.documentoged = :id
        """

        # Executando a query
        resp = self._db.execute_query(sql, id=id)

        # Checking if ID was found
        if len(resp) <= 0:
            raise NotFoundException(f"Anexo com id {id} não encontrado.")

        # Verificando se foi encontrado mais de um registro para o ID passado
        if len(resp) > 1:
            raise ConflictException(f"Encontrado mais de um Anexo, para o id {id}.")

        return resp[0]

    def list(
        self,
        grupo_anexos: uuid.UUID,
        modulo: int,
        tipo: int,
        mimetype: str,
        data: datetime.datetime,
    ):
        # Montando os filtros
        sql_where = "true"

        if grupo_anexos is not None:
            sql_where += "\nand am.id_modulodoanexo = :grupo_anexos"

        if modulo is not None:
            sql_where += "\nand am.modulodoanexo = :modulo"

        if tipo is not None:
            sql_where += "\nand am.tipo = :tipo"

        if mimetype is not None:
            sql_where += "\nand dg.mimetype = :mimetype"

        if data is not None:
            sql_where += "\nand dg.data = :data"

        # Motando a query principal
        sql = f"""
        with anexo_modulo_por_doc as (
            select
                am.documentoged, max(am.anexomodulo::text)::uuid as anexomodulo
            from
                ns.anexosmodulos as am
            group by
                am.documentoged
        )
        select
            dg.data, dg.nome, dg.documento, dg.mimetype, dg.documentoged, am.descricao, am.tipo, am.id_modulodoanexo, am.modulodoanexo
        from
            ns.documentosged as dg
            left join anexo_modulo_por_doc as amd on (amd.documentoged = dg.documentoged)
            left join ns.anexosmodulos as am on (am.anexomodulo = amd.anexomodulo)
        where
            {sql_where}
        """

        return self._db.execute_query(
            sql,
            grupo_anexos=grupo_anexos,
            modulo=modulo,
            tipo=tipo,
            mimetype=mimetype,
            data=data,
        )

    def inserir_documento_ged(
        self,
        id: uuid.UUID,
        nome: str,
        conteudo: str,
        mimetype: str,
        data: datetime.datetime,
    ) -> str:
        self._db.execute(
            "INSERT INTO ns.documentosged (data, nome, documento, mimetype, documentoged) VALUES (:data, :nome, :conteudo, :mimetype, :id)",
            data=data,
            nome=nome,
            conteudo=conteudo,
            mimetype=mimetype,
            id=id,
        )

    def inserir_anexo_modulo(
        self,
        nome: str,
        descricao: str,
        id_modulodoanexo: str,
        documento_ged_id: str,
        tipo: int,
        modulo_anexo: int,
    ) -> str:
        anexo_modulo_id = str(uuid.uuid4())
        self._db.execute(
            """INSERT INTO ns.anexosmodulos (anexomodulo, arquivo, descricao, modulodoanexo, id_modulodoanexo, documentoged, tipo)
            VALUES (:anexomodulo, :arquivo, :descricao, :modulodoanexo, :id_modulodoanexo, :documentoged, :tipo)""",
            anexomodulo=anexo_modulo_id,
            arquivo=nome,
            descricao=descricao,
            modulodoanexo=modulo_anexo,
            id_modulodoanexo=id_modulodoanexo,
            documentoged=documento_ged_id,
            tipo=tipo,
        )
        return anexo_modulo_id

    def inserir_pendencia(
        self,
        documento_ged_id: str,
    ) -> str:
        self._db.execute(
            """INSERT INTO ns.documentospendentesged (documentoged, data, tipopendencia)
            VALUES (:documentoged, :data, 1)""",
            documentoged=documento_ged_id,
            data=datetime.datetime.now(),
        )

    def delete_documento_ged(
        self,
        id: uuid.UUID,
    ) -> str:
        self._db.execute("delete from ns.documentosged where documentoged = :id", id=id)

    def delete_anexo_modulo(
        self,
        documento_ged_id: uuid.UUID,
    ) -> str:
        self._db.execute(
            "delete from ns.anexosmodulos where documentoged=:documento_ged_id",
            documento_ged_id=documento_ged_id,
        )

    def delete_pendencia(
        self,
        documento_ged_id: str,
    ) -> str:
        self._db.execute(
            """delete from ns.documentospendentesged where documentoged=:documentoged""",
            documentoged=documento_ged_id,
        )

    def update_documento_ged(
        self,
        id: uuid.UUID,
        nome: str,
        conteudo: str,
        mimetype: str,
        data: datetime.datetime,
        partial_update: bool = False,
    ) -> str:
        dados = {
            "data": data,
            "nome": nome,
            "conteudo": conteudo,
            "mimetype": mimetype,
            "id": id,
        }

        # Montando a parte dos campos no SQL
        sql_fields_set = []
        if not partial_update or data is not None:
            sql_fields_set.append("data=:data")
        if not partial_update or nome is not None:
            sql_fields_set.append("nome=:nome")
        if not partial_update or conteudo is not None:
            sql_fields_set.append("documento=:conteudo")
        if not partial_update or mimetype is not None:
            sql_fields_set.append("mimetype=:mimetype")

        if len(sql_fields_set) <= 0:
            raise Exception(
                "Ao menos uma das propriedades deve ser passada para uma atualização parcial (PATCH)."
            )

        sql_fields_set = ", ".join(sql_fields_set)

        # Executando o update
        self._db.execute(
            f"update ns.documentosged set {sql_fields_set} where documentoged=:id",
            **dados,
        )

    def update_anexo_modulo(
        self,
        id: uuid.UUID,
        nome: str,
        descricao: str,
        id_modulodoanexo: str,
        documento_ged_id: str,
        tipo: int,
        modulo_anexo: int,
        partial_update: bool = False,
    ) -> str:
        dados = {
            "id": id,
            "nome": nome,
            "descricao": descricao,
            "id_modulodoanexo": id_modulodoanexo,
            "documento_ged_id": documento_ged_id,
            "tipo": tipo,
            "modulo_anexo": modulo_anexo,
        }

        # Montando a parte dos campos no SQL
        sql_fields_set = []
        if not partial_update or nome is not None:
            sql_fields_set.append("arquivo=:nome")
        if not partial_update or descricao is not None:
            sql_fields_set.append("descricao=:descricao")
        if not partial_update or id_modulodoanexo is not None:
            sql_fields_set.append("id_modulodoanexo=:id_modulodoanexo")
        if not partial_update or documento_ged_id is not None:
            sql_fields_set.append("documentoged=:documento_ged_id")
        if not partial_update or tipo is not None:
            sql_fields_set.append("tipo=:tipo")
        if not partial_update or modulo_anexo is not None:
            sql_fields_set.append("modulodoanexo=:modulo_anexo")

        if len(sql_fields_set) <= 0:
            raise Exception(
                "Ao menos uma das propriedades deve ser passada para uma atualização parcial (PATCH)."
            )

        sql_fields_set = ", ".join(sql_fields_set)

        # Executando o update
        self._db.execute(
            f"update ns.anexosmodulos set {sql_fields_set} where anexomodulo=:id",
            **dados,
        )



import datetime
import uuid

from enum import Enum

from nsj_rest_lib.decorator.dto import DTO
from nsj_rest_lib.descriptor.dto_field import DTOField
from nsj_rest_lib.dto.dto_base import DTOBase


class TipoAnexo(Enum):
    GENERICO = ("generico", 0)
    NFE_XML = ("nfe_xml", 1)
    NFE_DANFE = ("nfe_danfe", 2)
    NFSE_XML = ("nfse_xml", 3)
    NFSE_PDF = ("nfse_pdf", 4)
    BOLETO_PDF = ("boleto_pdf", 5)
    BOLETO_REMESSA = ("boleto_remessa", 6)

    @classmethod
    def from_int_value(cls, int_value):
        for item in cls:
            if item.value[1] == int(int_value):
                return item
        raise ValueError(f"TipoAnexo {int_value} inválido.")


class ModuloAnexo(Enum):
    # Desktop
    BEM_PATRIMONIAL = ("bem_patrimonial", 0)
    CONTRATO = ("contrato", 1)
    OS_RPS_TITULO = ("os_rps_titulo", 2)
    ORDEM_SERVICO = ("ordem_servico", 3)
    REQUISICAO_COMPRA = ("requisicao_compra", 4)
    NEGOCIO = ("negocio", 5)
    DOCUMENTO_FISCAL = ("documento_fiscal", 6)
    ESTABELECIMENTO = ("estabelecimento", 7)
    # Web
    ATENDIMENTO = ("atendimento", 1001)
    FOLLOWUP = ("followup", 1002)
    ATIVIDADE = ("atividade", 1003)


@DTO()
class AnexoDTO(DTOBase):
    id: uuid.UUID = DTOField(
        pk=True, resume=True, default_value=uuid.uuid4, entity_field="documentoged"
    )  # description="Conteúdo do anexo em base64"
    url: str = DTOField(resume=True)  # description="URL para download do anexo"
    conteudo: str = DTOField()  # description="Conteúdo do anexo em base64"
    conteudo_binario: bytes = DTOField()
    nome: str = DTOField(
        not_null=True, resume=True, max=255, strip=True
    )  # description="Nome do anexo"
    descricao: str = DTOField(
        resume=True, max=255, strip=True
    )  # description="Descrição do anexo"
    mimetype: str = DTOField(
        resume=True, max=255, strip=True
    )  # description="MIME type do anexo"
    tipo: TipoAnexo = DTOField(
        resume=True, default_value=TipoAnexo.GENERICO
    )  # description="Tipo do anexo"
    data: datetime.datetime = DTOField(
        default_value=datetime.datetime.now, resume=True
    )  # description="Tipo do anexo"
    grupo_anexos: uuid.UUID = DTOField(
        not_null=True, resume=True, entity_field="id_modulodoanexo"
    )  # description="ID do grupo de anexos ao qual este anexo pertence"
    modulo: ModuloAnexo = DTOField(
        not_null=True, resume=True, entity_field="modulodoanexo"
    )  # description="Enum indicando o módulo do ERP, ao qual o anexo pertence"



import io
import logging
import mimetypes
import os
import uuid

from flask import send_file

from nsj_gcf_utils.db_adapter2 import DBAdapter2
from nsj_gcf_utils.rest_error_util import format_json_error
from nsj_rest_lib.controller.controller_util import DEFAULT_RESP_HEADERS

from erp3_py_commons.anexo.anexo_dao import AnexoDAO
from erp3_py_commons.anexo.anexo_service import AnexoService


def get_logger():
    APP_NAME = os.getenv("APP_NAME", "erp3-py-commons")
    return logging.getLogger(APP_NAME)


def download_anexo(db: DBAdapter2, id: uuid.UUID):

    try:
        # Recuperando o nexo pelo ID
        dao = AnexoDAO(db)
        service = AnexoService(dao)

        # Conteúdo do arquivo em bytes (pode variar dependendo do ID)
        anexo = service.get(id, None, None, conteudo_em_bytes=True)

        # Crie um objeto BytesIO
        byte_io = io.BytesIO()
        byte_io.write(anexo.conteudo_binario)
        byte_io.seek(0)

        # Resolvendo o mimetype
        mimetype = anexo.mimetype
        if mimetype is None:
            mimetype, _ = mimetypes.guess_type(anexo.nome)

        # Envie o arquivo
        return send_file(
            byte_io,
            as_attachment=True,
            download_name=anexo.nome,
            mimetype=mimetype,
        )
    except Exception as e:
        get_logger().exception(e)
        return (
            format_json_error(f"Erro desconhecido: {e}"),
            500,
            {**DEFAULT_RESP_HEADERS},
        )



import base64
import os
import re
import uuid

from flask import request
from typing import Any, Callable, Dict, List, Set

from nsj_rest_lib.service.service_base import ServiceBase

from erp3_py_commons.anexo.anexo_dto import AnexoDTO
from erp3_py_commons.anexo.anexo_dao import AnexoDAO

from nsj_rest_lib.dto.dto_base import DTOBase
from nsj_rest_lib.entity.entity_base import EntityBase
from nsj_rest_lib.injector_factory_base import NsjInjectorFactoryBase


class AnexoService(ServiceBase):
    _dao: AnexoDAO

    def __init__(
        self,
        dao: AnexoDAO,
    ):
        self._dao = dao
        self.control_transaction = True

    def disable_control_transaction(self):
        self.control_transaction = False

    def get(
        self,
        id: str,
        partition_fields: Dict[str, Any],
        fields: Dict[str, Set[str]],
        conteudo_em_bytes: bool = False,
    ) -> DTOBase:
        model = self._dao.get(id)

        # Tratando da codificação do conteúdo em Base64
        conteudo = model["documento"]
        if conteudo is not None:
            if not conteudo_em_bytes:
                conteudo = base64.b64encode(conteudo).decode("utf-8")
                conteudo_bytes = None
            else:
                conteudo_bytes = conteudo
                conteudo = None
        del model["documento"]

        # Montando a URL de download
        url = self._get_url_download_anexo(model["documentoged"])

        # Construindo o DTO de retorno
        return AnexoDTO(
            **model,
            conteudo=conteudo,
            conteudo_binario=conteudo_bytes,
            kwargs_as_entity=True,
            url=url,
        )

    def _get_url_download_anexo(self, id: uuid.UUID, tenant: int = None):
        # Getting URL base from request
        if os.getenv("ENV", "").lower() != "erp_sql":
            base_url = request.base_url

            if tenant is None:
                if "tenant" in request.args:
                    tenant = request.args["tenant"]
                else:
                    raise Exception("Faltando parâmetro 'tenant' na requisição.")
        else:
            base_url = ""

        # Deixando apenas as duas primeiras partes da URL base
        if len(base_url) > 0:
            match = re.match("(https?://[^/]+/[^/]+)(/.*)?", base_url)
            if match is not None:
                base_url = match.group(1)

            if base_url[-1] != "/":
                base_url = f"{base_url}/"

        # Getting URL from download atach endpoint
        download_attach = os.getenv("URL_DOWNLOAD_ANEXOS", "")

        if len(download_attach) > 0 and download_attach[0] == "/":
            download_attach = download_attach[1:]

        if len(download_attach) > 0 and download_attach[-1] == "/":
            download_attach = download_attach[:-1]

        # Making final URL
        return f"{base_url}{download_attach}/{id}?tenant={tenant}"

    def list(
        self,
        after: uuid.UUID,
        limit: int,
        fields: Dict[str, Set[str]],
        order_fields: List[str],
        filters: Dict[str, Any],
        search_query: str = None,
    ) -> List[DTOBase]:

        # Tratando dos filtros
        grupo_anexos = None
        if "grupo_anexos" in filters:
            grupo_anexos = filters["grupo_anexos"]

        modulo = None
        if "modulo" in filters:
            modulo = filters["modulo"]

        tipo = None
        if "tipo" in filters:
            tipo = filters["tipo"]

        mimetype = None
        if "mimetype" in filters:
            mimetype = filters["mimetype"]

        data = None
        if "data" in filters:
            data = filters["data"]

        # Retrieving from DAO
        model_list = self._dao.list(grupo_anexos, modulo, tipo, mimetype, data)

        # Convertendo para uma lista de DTOs
        dto_list = []
        for model in model_list:
            # Tratando da codificação do conteúdo em Base64
            conteudo = model["documento"]
            if conteudo is not None:
                conteudo = base64.b64encode(conteudo).decode(encoding="utf-8")
            del model["documento"]

            # Montando a URL de download
            url = self._get_url_download_anexo(model["documentoged"])

            # Criando o DTO
            dto = AnexoDTO(**model, conteudo=conteudo, kwargs_as_entity=True, url=url)

            # Adicionando o DTO na lista
            dto_list.append(dto)

        # Returning
        return dto_list

    def insert(
        self,
        dto: AnexoDTO,
        aditional_filters: Dict[str, Any] = None,
        custom_before_insert: Callable = None,
        custom_after_insert: Callable = None,
    ) -> DTOBase:
        try:
            start_transaction = False
            if self.control_transaction and not self._dao._db.in_transaction():
                start_transaction = True
                self._dao._db.begin()

            # Inserindo na tabela documentosged
            conteudo_decodificado = dto.conteudo
            if conteudo_decodificado is not None:
                conteudo_decodificado = base64.b64decode(dto.conteudo)

            self._dao.inserir_documento_ged(
                dto.id,
                dto.nome,
                conteudo_decodificado,
                dto.mimetype,
                dto.data,
            )

            # Inserindo na tabela anexosmodulos
            self._dao.inserir_anexo_modulo(
                dto.nome,
                dto.descricao,
                dto.grupo_anexos,
                dto.id,
                dto.tipo.value[1],
                dto.modulo.value[1],
            )

            # Inserindo na tabela de pendencias
            self._dao.inserir_pendencia(dto.id)

            if start_transaction:
                self._dao._db.commit()

            return None
        finally:
            if start_transaction:
                self._dao._db.rollback()

    def update(
        self,
        dto: AnexoDTO,
        id: Any,
        aditional_filters: Dict[str, Any] = None,
        custom_before_update: Callable = None,
        custom_after_update: Callable = None,
    ) -> DTOBase:
        self._update(dto, id, False)
        return None

    def _update(
        self,
        dto: AnexoDTO,
        id: Any,
        partial_update: bool,
    ):

        try:
            start_transaction = False
            if self.control_transaction and not self._dao._db.in_transaction():
                start_transaction = True
                self._dao._db.begin()

            # Atualizando a tabela documentosged
            conteudo_decodificado = dto.conteudo
            if conteudo_decodificado is not None:
                conteudo_decodificado = base64.b64decode(dto.conteudo)

            self._dao.update_documento_ged(
                id,
                dto.nome,
                conteudo_decodificado,
                dto.mimetype,
                dto.data,
                partial_update=partial_update,
            )

            # Recuperando o ID da tabela anexosmodulos
            grupo_anexos = dto.grupo_anexos
            if grupo_anexos is None:
                dto_old = self.get(id)
                grupo_anexos = dto_old.grupo_anexos

            # Atualizando na tabela anexosmodulos
            self._dao.update_anexo_modulo(
                grupo_anexos,
                dto.nome,
                dto.descricao,
                dto.grupo_anexos,
                dto.id,
                dto.tipo.value if dto.tipo is not None else None,
                dto.modulo.value if dto.modulo is not None else None,
                partial_update=partial_update,
            )

            if start_transaction:
                self._dao._db.commit()
        finally:
            if start_transaction:
                self._dao._db.rollback()

    def partial_update(
        self,
        dto: DTOBase,
        id: Any,
        aditional_filters: Dict[str, Any] = None,
        custom_before_update: Callable = None,
        custom_after_update: Callable = None,
    ) -> DTOBase:
        self._update(dto, id, True)
        return None

    def delete(self, id: Any, additional_filters: Dict[str, Any] = None) -> DTOBase:
        try:
            start_transaction = False
            if self.control_transaction and not self._dao._db.in_transaction():
                start_transaction = True
                self._dao._db.begin()

            # Apagando da tabela de pendencias
            self._dao.delete_pendencia(id)

            # Apagando da tabela anexosmodulos
            self._dao.delete_anexo_modulo(id)

            # Apagando da tabela documentosged
            self._dao.delete_documento_ged(id)

            if start_transaction:
                self._dao._db.commit()

            return None
        finally:
            if start_transaction:
                self._dao._db.rollback()
